#ifndef MRK_PARSER_INTERNAL
#define MRK_PARSER_INTERNAL

#include "mrk/lexer_internal.h"
#include "mrk/parser.h"

#define MRK_PARSER_LOOKAHEAD_BUF_SIZE 4
#define MRK_PARSE_ERR(p, t, m)                                                 \
  p->error.token = t;                                                          \
  p->error.msg = m;                                                            \
  return mrk_err_invalid

struct mrk_parser {
  mrk_lexer *lexer;
  struct {
    mrk_token buf[MRK_PARSER_LOOKAHEAD_BUF_SIZE];
    size_t index;
  } lookahead;
  struct {
    mrk_token token;
    const char *msg;
  } error;
};

/**
 * Returns true if all the data from the lexer has been processed.
 *
 * Because the parser might buffer parts of the results of the lexer, this is
 * not equivalent to calling lexer done.
 */
bool mrk_parser_done(mrk_parser *parser);

/**
 * Return the next token that would be consumed by a call to advance.
 */
mrk_token mrk_parser_peek(mrk_parser *parser);

/**
 * Advance the processing stream by one token.
 */
void mrk_parser_advance(mrk_parser *parser);

/**
 * Advance the parser only if the newly consumed token would be the one
 * specified as the argument. Otherwise, throw an error.
 */
mrk_err mrk_parser_eat(mrk_token *out, mrk_parser *parser, mrk_token_type type);

mrk_err mrk_parser_parse_block(mrk_ast_node *out, mrk_parser *parser);

/**
 * Parse an entire header block.
 */
mrk_err mrk_parser_parse_header(mrk_ast_node *out, mrk_parser *parser);

/**
 * Parse a paragraph. Note that a paragraph does not necessarily end with a
 * blank line, but rather when it either encounters a blank line, end of file,
 * or some other token that indicates a new block should start (e.g. start of a
 * list).
 */
mrk_err mrk_parser_parse_paragraph(mrk_ast_node *out, mrk_parser *parser);

/**
 * Parse a text token, as well as any following text tokens, delimited by a
 * single newline which will be converted to a space.
 */
mrk_err mrk_parser_parse_text(mrk_ast_node *out, mrk_parser *parser);

/**
 * Parse a link construct
 */
mrk_err mrk_parser_parse_link(mrk_ast_node *out, mrk_parser *parser);

#endif
