#include "mrk/parser_internal.h"

bool mrk_parser_done(mrk_parser *parser) {
  return mrk_lexer_done(parser->lexer) &&
         parser->lookahead.buf[parser->lookahead.index].type ==
             mrk_token_type_none;
}

mrk_token mrk_parser_peek(mrk_parser *parser) {
  mrk_token out;

  if (mrk_parser_done(parser)) {
    out.type = mrk_token_type_none;
  } else {
    out = parser->lookahead.buf[parser->lookahead.index];
  }

  return out;
}

void mrk_parser_advance(mrk_parser *parser) {
  if (mrk_parser_done(parser)) {
    return;
  }

  // Append new element of the lexer to the lookahead buffer
  if (!mrk_lexer_done(parser->lexer)) {
    mrk_lexer_next(&parser->lookahead.buf[parser->lookahead.index],
                   parser->lexer);
  }

  parser->lookahead.index =
      (parser->lookahead.index + 1) % MRK_PARSER_LOOKAHEAD_BUF_SIZE;
}

mrk_err mrk_parser_eat(mrk_token *out, mrk_parser *parser,
                       mrk_token_type type) {
  if (mrk_parser_done(parser) || mrk_parser_peek(parser).type != type) {
    parser->error.msg = "Unexpected token";
    parser->error.token = mrk_parser_peek(parser);

    return mrk_err_unexpected_token;
  }

  if (out != NULL) {
    *out = mrk_parser_peek(parser);
  }

  mrk_parser_advance(parser);

  return mrk_err_ok;
}
